(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(require('babel-runtime/regenerator'), require('babel-runtime/core-js/promise'), require('babel-runtime/helpers/asyncToGenerator'), require('babel-runtime/helpers/classCallCheck'), require('babel-runtime/helpers/createClass'), require('scxml'), require('vm'), require('babel-runtime/core-js/object/keys'), require('babel-runtime/core-js/get-iterator'), require('babel-runtime/core-js/json/stringify'), require('request'), require('debug'), require('moment-business-days'), require('date-utils')) :
    typeof define === 'function' && define.amd ? define(['babel-runtime/regenerator', 'babel-runtime/core-js/promise', 'babel-runtime/helpers/asyncToGenerator', 'babel-runtime/helpers/classCallCheck', 'babel-runtime/helpers/createClass', 'scxml', 'vm', 'babel-runtime/core-js/object/keys', 'babel-runtime/core-js/get-iterator', 'babel-runtime/core-js/json/stringify', 'request', 'debug', 'moment-business-days', 'date-utils'], factory) :
    (factory(global._regeneratorRuntime,global._Promise,global._asyncToGenerator,global._classCallCheck,global._createClass,global.scxml,global.vm,global._Object$keys,global._getIterator,global._JSON$stringify,global.request,global.debugStart,global.moment,global.dateUtils));
}(this, (function (_regeneratorRuntime,_Promise,_asyncToGenerator,_classCallCheck,_createClass,scxml,vm,_Object$keys,_getIterator,_JSON$stringify,request,debugStart,moment,dateUtils) { 'use strict';

_regeneratorRuntime = 'default' in _regeneratorRuntime ? _regeneratorRuntime['default'] : _regeneratorRuntime;
_Promise = 'default' in _Promise ? _Promise['default'] : _Promise;
_asyncToGenerator = 'default' in _asyncToGenerator ? _asyncToGenerator['default'] : _asyncToGenerator;
_classCallCheck = 'default' in _classCallCheck ? _classCallCheck['default'] : _classCallCheck;
_createClass = 'default' in _createClass ? _createClass['default'] : _createClass;
scxml = 'default' in scxml ? scxml['default'] : scxml;
vm = 'default' in vm ? vm['default'] : vm;
_Object$keys = 'default' in _Object$keys ? _Object$keys['default'] : _Object$keys;
_getIterator = 'default' in _getIterator ? _getIterator['default'] : _getIterator;
_JSON$stringify = 'default' in _JSON$stringify ? _JSON$stringify['default'] : _JSON$stringify;
request = 'default' in request ? request['default'] : request;
debugStart = 'default' in debugStart ? debugStart['default'] : debugStart;
moment = 'default' in moment ? moment['default'] : moment;

/**
 * Created by Ricardo Morais on 07/06/2017.
 */

var debug$2 = require("debug")("interpreter-actions");
var vm$1 = require('vm');
var scheduleLib = require('node-schedule');
var vsprintf = require('sprintf-js').vsprintf;

var jobs = {};
/**
 * Schedule an event for a specific date
 * <schedule event="eventToRaise" exprDate="new Date()" job="myJob"/>
 * @param arguments
 * @param sandbox
 */
function schedule(args, sandbox, eventContext) {
    debug$2("Action schedule");
    if (!args.raise) {
        throw new Error("Missing the raise argument, not executed");
    }
    var event = args.raise;
    var date = args.date;
    var jobIdentifier = args.job;

    var job = scheduleLib.scheduleJob(date, function () {
        debug$2("Fired '%s' event on scheduled date '%s'", event, date);
        if (jobIdentifier) {
            delete jobs[jobIdentifier];
        }
        this.send({ name: event });
    }.bind(this));

    if (args.job) {
        jobs[args.job] = job;
    }
}

/**
 * Unschedules a job
 * <unschedule job="myJob"/>
 * @param arguments
 * @param sandbox
 * @param event
 */
function unschedule(args, sandbox, eventContext) {
    debug$2("Action unschedule");
    if (!args.job) {
        throw new Error("Missing the job argument, not executed");
    }
    jobs[args.job].cancel();
}

function log(args, sandbox, eventContext, engine) {
    debug$2("Action log");

    if (!args.message) {
        throw new Error("Missing the message argument, not executed");
    }

    args.data = args.data || [];

    console.log(vsprintf(args.message, args.data));
}

function runInstance(args, sandbox, eventContext, engine) {
    var _this = this;

    debug$2("Action runInstance");

    if (!args.machine || !args.versionKey) {
        throw new Error("Missing the machine or versionKey or instanceKey arguments, not executed");
    }

    engine.addInstance(args.machine, args.versionKey).then(function (instanceKey) {
        engine.runInstance(args.machine, args.versionKey, instanceKey).then(function (job) {
            job.on('complete', function (result) {
                _this.send({
                    name: args.raise || 'runningInstance',
                    machine: args.machine,
                    versionKey: args.versionKey,
                    instanceKey: instanceKey
                });
            });
        });
    });
}

function stopInstance(args, sandbox, eventContext, engine) {
    var _this2 = this;

    debug$2("Action stopInstance");

    if (!args.machine || !args.versionKey || !args.instanceKey) {
        throw new Error("Missing the machine or versionKey or instanceKey arguments, not executed");
    }

    engine.stopInstance(args.machine, args.versionKey, args.instanceKey).then(function (job) {
        job.on('complete', function (result) {
            _this2.send({
                name: args.raise || 'runningInstance',
                machine: args.machine,
                versionKey: args.versionKey,
                instanceKey: data.instanceKey
            });
        });
    });
}

function sendEvent$1(args, sandbox, eventContext, engine) {
    debug$2("Action sendEvent");

    if (!args.machine || !args.versionKey || !args.instanceKey) {
        throw new Error("Missing the machine or versionKey or instanceKey arguments, not executed");
    }

    if (!args.raise) {
        throw new Error("Missing the raise argument, not executed");
    }

    engine.sendEvent(args.machine, args.versionKey, args.instanceKey, args.raise || 'eventSent', args.data).then();
}

var engineActions = {
    schedule: schedule,
    unschedule: unschedule,
    log: log,
    runInstance: runInstance,
    stopInstance: stopInstance,
    sendEvent: sendEvent$1
};

/**
 * Created by Ricardo Morais on 28/03/2017.
 */

var debug$1 = debugStart("custom-action");

var CustomExecutableContent = function () {
    function CustomExecutableContent(engine) {
        var dispatcherURL = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
        var dispatcherToken = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

        _classCallCheck(this, CustomExecutableContent);

        this.engine = engine;
        this.dispatcherURL = dispatcherURL;
        this.dispatcherToken = dispatcherToken;
    }

    /**
     * Swaps the dispatcher
     * @param dispatcherURL The new Dispatcher URL
     * @param dispatcherToken The new Dispatcher Token
     */


    _createClass(CustomExecutableContent, [{
        key: "swapDispatcher",
        value: function swapDispatcher(dispatcherURL, dispatcherToken) {
            this.dispatcherURL = dispatcherURL;
            this.dispatcherToken = dispatcherToken;
        }

        /**
         * Executes a custom action on an actions server host by calling its REST API /execute method
         * @param context The execution context of the statechart
         * @param namespace The namespace of the action
         * @param action The name of the action
         * @param sandbox The sandbox of the finite-state machine instance which the action was requested
         * @param event The event in case the action was requested inside a transition
         * @param actionArguments The arguments of the action call
         */

    }, {
        key: "execute",
        value: function execute(context, namespace, action, sandbox, event, actionArguments) {

            try {
                var data = {
                    namespace: namespace,
                    action: action,
                    arguments: {}
                };

                debug$1("Custom action with data: %s", _JSON$stringify(data));

                var _iteratorNormalCompletion = true;
                var _didIteratorError = false;
                var _iteratorError = undefined;

                try {
                    for (var _iterator = _getIterator(_Object$keys(actionArguments)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                        var key = _step.value;

                        if (key[0] !== '$') {
                            if (key.startsWith("expr")) {
                                var newKey = key.substring("expr".length);
                                newKey = newKey[0].toLowerCase() + newKey.substring(1);
                                var prefix = 'var _event = ' + _JSON$stringify(event) + ';';
                                vm.runInContext(prefix, sandbox);
                                vm.runInContext("var result6990080817 = " + actionArguments[key], sandbox);
                                data.arguments[newKey] = sandbox.result6990080817;
                            } else {
                                data.arguments[key] = actionArguments[key];
                            }
                        }
                    }
                } catch (err) {
                    _didIteratorError = true;
                    _iteratorError = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion && _iterator.return) {
                            _iterator.return();
                        }
                    } finally {
                        if (_didIteratorError) {
                            throw _iteratorError;
                        }
                    }
                }

                if (namespace.toLowerCase() === "https://insticc.org/fsm-engine") {
                    if (typeof engineActions[action] === 'function') {
                        engineActions[action].call(context, data.arguments, sandbox, event, this.engine);
                    }
                    return;
                }

                if (!this.dispatcherURL) {
                    return;
                }

                var host = this.dispatcherURL;
                var route = "/execute";
                var errorEvent = data.arguments.errorEvent;
                var successEvent = data.arguments.successEvent;

                request({
                    url: host + route,
                    headers: {
                        'Authorization': 'JWT ' + this.dispatcherToken
                    },
                    method: "POST",
                    json: true,
                    body: data
                }, function (error, response, body) {
                    if (error) {
                        debug$1("Request to action dispatcher failed with error: %s", error);
                        if (errorEvent) {
                            context.send({ name: errorEvent, data: error });
                        }
                        return;
                    }
                    if (successEvent) {
                        debug$1("Request to action dispatcher was successful");
                        context.send({ name: successEvent, data: error });
                    }
                });
            } catch (err) {
                debug$1("Custom action not executed", err);
            }
        }
    }]);

    return CustomExecutableContent;
}();

/**
 * Created by Ricardo Morais on 17/06/2017.
 */

//Libraries
var debug = debugStart("interpreter");
var REFRESH_INTERVAL = 1000;

/**
 * Represents a SCXML interpreter
 */

var Interpreter = function () {

    /**
     * @constructor
     * @param engine
     * @param {String} documentString The SCXML
     * @param {String|null} actionDispatcherURL An optional action dispatcher URL
     * @param {string|null} actionDispatcherToken An optional access token for the action dispatcher
     * @param {string} machine The machine name
     * @param {string} versionKey The versionKey
     * @param {string} instanceKey The instanceKey
     */
    function Interpreter(engine, machine, versionKey, instanceKey, documentString) {
        _classCallCheck(this, Interpreter);

        this.engine = engine;
        this.machine = machine;
        this.versionKey = versionKey;
        this.instanceKey = instanceKey;
        this.documentString = documentString;
        this.customExecContent = new CustomExecutableContent(engine, engine.dispatcherURL, engine.dispatcherToken);
        this.hasChanged = false;
    }

    /**
     * Swaps the dispatcher
     * @param dispatcherURL The new Dispatcher URL
     * @param dispatcherToken The new Dispatcher Token
     */


    _createClass(Interpreter, [{
        key: "swapDispatcher",
        value: function swapDispatcher(dispatcherURL, dispatcherToken) {
            this.dispatcherURL = dispatcherURL;
            this.dispatcherToken = dispatcherToken;
            this.customExecContent.swapDispatcher(dispatcherURL, dispatcherToken);
        }
        /**
         * Create the sandbox for the interpreter
         * @method _createSandbox
         * @memberOf Interpreter
         * @private
         */

    }, {
        key: "_createSandbox",
        value: function _createSandbox() {

            var custom = this.customExecContent;
            //Define the sandbox for the v8 virtual machine
            var sandbox = {
                moment: moment,
                Date: Date,
                me: {
                    machine: this.machine,
                    versionKey: this.versionKey,
                    instanceKey: this.instanceKey
                },
                //The server global variables and functions
                // globals: serverGlobal,
                //The function that will process the custom actions
                postMessage: function postMessage(message) {
                    var type = message.data["$type"];
                    var stripNsPrefixRe = /^(?:{(?:[^}]*)})?(.*)$/;
                    var arr = stripNsPrefixRe.exec(type);
                    var ns = void 0;
                    var action = void 0;
                    if (arr.length === 2) {
                        ns = type.substring(1, type.indexOf("}"));
                        action = arr[1];
                    } else {
                        ns = "";
                        action = arr[0];
                    }

                    custom.execute(this, ns, action, sandbox, message._event, message.data);
                }
            };

            return vm.createContext(sandbox);
        }

        /**
         * Start the interpreter
         * @method start
         * @memberOf Interpreter
         * @returns {Promise}
         */

    }, {
        key: "start",
        value: function () {
            var _ref = _asyncToGenerator(_regeneratorRuntime.mark(function _callee(snapshot) {
                var _this = this;

                var model, vmSandbox, fnModel;
                return _regeneratorRuntime.wrap(function _callee$(_context) {
                    while (1) {
                        switch (_context.prev = _context.next) {
                            case 0:
                                debug("Starting the interpreter");

                                //Create the SCION model from the SCXML document
                                _context.next = 3;
                                return new _Promise(function (resolve, reject) {
                                    scxml.documentStringToModel(null, _this.documentString, function (err, model) {
                                        if (err) {
                                            reject(err);
                                        }
                                        resolve(model);
                                    });
                                }).then();

                            case 3:
                                model = _context.sent;
                                vmSandbox = this._createSandbox();

                                //Create the SCION-CORE fnModel to use in its interpreter

                                _context.next = 7;
                                return new _Promise(function (resolve, reject) {
                                    model.prepare(function (err, fnModel) {
                                        if (err) {
                                            reject(err);
                                        }
                                        resolve(fnModel);
                                    }, vmSandbox);
                                }).then();

                            case 7:
                                fnModel = _context.sent;


                                this.sandbox = vmSandbox;
                                //Instantiate the interpreter
                                this.sc = new scxml.scion.Statechart(fnModel, { snapshot: snapshot });
                                this.sc.start();

                                _context.next = 13;
                                return this.save();

                            case 13:

                                //Mark has changed
                                this.sc.on("onTransition", function () {
                                    _this.hasChanged = true;
                                });

                                this._hasStarted = true;
                                this.interval = setInterval(function () {
                                    if (_this.sc === null) {
                                        clearInterval(_this.interval);
                                        return;
                                    }
                                    if (_this.sc.isFinal()) {
                                        clearInterval(_this.interval);
                                        _this.save().then();
                                        return;
                                    }
                                    if (!_this.sc._isStepping && _this.hasChanged) {
                                        _this.save().then();
                                        _this.hasChanged = false;
                                    }
                                }, REFRESH_INTERVAL);

                            case 16:
                            case "end":
                                return _context.stop();
                        }
                    }
                }, _callee, this);
            }));

            function start(_x) {
                return _ref.apply(this, arguments);
            }

            return start;
        }()
    }, {
        key: "save",
        value: function () {
            var _ref2 = _asyncToGenerator(_regeneratorRuntime.mark(function _callee2() {
                return _regeneratorRuntime.wrap(function _callee2$(_context2) {
                    while (1) {
                        switch (_context2.prev = _context2.next) {
                            case 0:
                            case "end":
                                return _context2.stop();
                        }
                    }
                }, _callee2, this);
            }));

            function save() {
                return _ref2.apply(this, arguments);
            }

            return save;
        }()
    }, {
        key: "getSnapshot",
        value: function getSnapshot() {
            return this.sc.getSnapshot();
        }
    }, {
        key: "onFinish",
        value: function onFinish() {}
        // override


        /**
         * Send an event to the statechart
         * @method sendEvent
         * @memberOf Interpreter
         * @param {String} event The name of the event
         * @param {Object} eventData The data of the event
         */

    }, {
        key: "sendEvent",
        value: function sendEvent(event, eventData) {
            //Find out if the instance has already started
            if (!this._hasStarted) {
                throw new Error("Can't send event, the interpreter hasn't started");
            }

            var data = eventData;
            data.event = event;

            this.sc.gen(data);
            this.save().then();
        }
    }]);

    return Interpreter;
}();

/**
 * Created by Ricardo Morais on 07/08/2017.
 */

module.exports = Interpreter;

})));
//# sourceMappingURL=index.umd.js.map
